;; jbank@alum.mit.edu
;; 10/2000
;;
;; This provides some simple buffer searching capabilities,
;; requested by paulk@arsdigita.com
;;
;; The basic idea is to do something similar to a tags search,
;; but using the text within your current buffers.
;;
;; Exports: search-my-buffers and continue-search-my-buffers
;; bindings: C-x M-. and C-x M-,


(require 'etags)
(defun current-sexp ()
  (find-tag-default))

(setq last-search-buffers)
(setq last-search-point)
(setq last-search-string)

(defun search-buffers-for-string (string buffers) 
  (if buffers
      (let ((buffer (car buffers)))
         
        (set-buffer buffer)
        (save-excursion 
          (let ((pt (if (eq buffers last-search-buffers)
                    last-search-point
                    (point-min))))
            (goto-char pt)
            (if (search-forward string (point-max) t)
                (progn (setq last-search-string string)
                       (setq last-search-buffers buffers)
                       (setq last-search-point (point))
                       (list buffer (point)))
              (search-buffers-for-string string (cdr buffers))))))))

(defun do-search (string buffers)
  (let ((buffer-info (search-buffers-for-string string buffers)))
    (if buffer-info
        (let ((buffer (car buffer-info))
              (pt (cadr buffer-info)))
          (switch-to-buffer buffer)
          (goto-char pt))
      (message "Search failed for: " string))))

(defun search-my-buffers (string)
  (interactive (list (read-from-minibuffer "Search: " (current-sexp))))
  (do-search string (buffer-list)))

(defun continue-search-my-buffers ()
  (interactive)
  (do-search last-search-string last-search-buffers))

(global-set-key "\C-x\M-." 'search-my-buffers)
(global-set-key "\C-x\M-," 'continue-search-my-buffers)

