;; jbank@alum.mit.edu
;; 10/2000
;;
;; Provide various support stuff for ACS Java development.
;; In particular sets up multi-mode stuff for editing jsp
;; files.
;;
;; Also provides some ACS 3.4 Java specific tag utility functions.

;;this is needed in multimode
(defun third (x)
  (car (cdr (cdr x))))

;;multi-mode
(autoload 'multi-mode
  "multi-mode"
  "Allowing multiple major modes in a buffer."
  t)

(defun jsp-mode () (interactive)
  (multi-mode 1
              'html-mode
              '("<%" java-mode)
              '("%>" html-mode)))

(setq auto-mode-alist
      (append '(("\\.jsp$" . jsp-mode)
                ("\\.tpl$" . jsp-mode)
                ("\\.ajp$" . jsp-mode))
            auto-mode-alist))

;;Some utilities for java-acs

(defun current-sexp ()
  (find-tag-default))

(defun insert-one-att (name val) 
  (insert " " name "=\"" val "\""))

(defun adsub (var-name source encode) 
  (interactive (list (read-from-minibuffer "Var name: " (current-sexp)) 
                     (read-from-minibuffer "datasource (empty uses default):")
                     (read-from-minibuffer "encode (empty uses default):")))
  (insert "<ad:sub ")
  (insert-one-att "var" var-name)
  (if (not (equal source ""))
      (insert-one-att "ds" source))
  (if (not (equal encode ""))
      (insert-one-att "encode" encode))
  (insert "/>"))

(defun insert-ad-query (tag id obj) 
  (insert "<" tag)
  (insert-one-att "id" id)
  (if (not (equal obj ""))
      (insert-one-att "obj" obj))
  (insert ">\n")
  (save-excursion (insert "\n</" tag ">")))

(defun addbquery (id obj) 
  (interactive (list (read-from-minibuffer "id: ") (read-from-minibuffer "obj: " "")))
  (insert-ad-query "ad:db_query" id obj))

(defun addb_0or1row (id obj) 
  (interactive (list (read-from-minibuffer "id: ") (read-from-minibuffer "obj: " "")))
  (insert-ad-query "ad:db_0or1row" id obj))

(defun addb_1row (id obj) 
  (interactive (list (read-from-minibuffer "id: ") (read-from-minibuffer "obj: " "")))
  (insert-ad-query "ad:db_1row" id obj))

(defun adscope (sources) 
  (interactive (list (read-from-minibuffer "sources: ")))
  (insert "<ad:scope sources=\"" sources "\">\n")
  (save-excursion (insert "\n</ad:scope>")))

(defun adloop (ds) 
  (interactive (list (read-from-minibuffer "ds: ")))
  (insert "<ad:loop ds=\"" ds "\">\n")
  (save-excursion (insert "\n</ad:loop>")))

(defun adexport (type source vars) 
  (interactive (list (read-from-minibuffer "Type: " "form") (read-from-minibuffer "Source: " "form") (read-from-minibuffer "Vars: " "return_url") ))
  (insert "<ad:export type=\"" type "\" source=\"" source "\" vars=\"" vars "\"/>"))


(defun per_eq (var-name) 
  (interactive (list (read-from-minibuffer "Var name: ")))
  (insert "<%= " var-name " %>"))

(defun per_eq_novar () 
  (interactive)
  (insert "<%= ")
  (forward-sexp)
  (insert " %>"))

(defun adsub_novar () 
  (interactive)
  (insert "<ad:sub var=\"")
  (forward-sexp)
  (insert "\"/>"))



(add-hook 'html-mode-hook (function (lambda ()
  (define-key html-mode-map "\C-x%" 'per_eq_novar)
  (define-key html-mode-map "\C-xa" 'adsub_novar))))


(setq font-lock-global-modes (append '(jsp-mode html-mode java-mode) (if (boundp 'font-lock-global-modes) font-lock-global-modes nil)))