;; jbank@alum.mit.edu
;; 10/2000
;;
;; Provides simple ability to type "C-c." from an 
;; include directive in a C or C++ file and get taken
;; directly to that include file.  Does its searching
;; based on the current environment value of INCLUDE.
;;
;; bindings: C-c .

(require 'joes-utils)

(if (boundp 'c++-mode-map)
    (define-key c++-mode-map "\C-c." 'show-include-file))

;;;###autoload
(defun show-include-file (&optional where)
  "Show the include file for the #include preprocessor directive on
  the current line.  Optional prefix argument WHERE is passed on to
  the find-file routine."
  (interactive "p")
  (let* ((line (save-excursion 
		(buffer-substring (progn (beginning-of-line) (point))
				  (progn (end-of-line) (point)))))
        (system-include (and (getenv "INCLUDE")
				 (string-split ";" (getenv "INCLUDE"))))
        (include-search-path (append (if (boundp 'include-search-path)
                                         include-search-path)
                                     system-include)))
    (or (string-match "^[ \t]*#[ \t]*include" line)
	(error "Current line is not a #include directive."))
    (let* ((rest (substring line (match-end 0) (length line)))
	   (incl
	     (cond
	      ((string-match "^[ \t]*<\\([^>]+\\)>" rest)
	       (substring rest (match-beginning 1) (match-end 1)))
	      ((string-match "^[ \t]*\"\\([^\"]+\\)\"" rest)
	       (substring rest (match-beginning 1) (match-end 1)))
	      (t
	       (error "Malformed or macro-expansion #include.")))))
      (let ((file (locate-file incl
			       (cons "." include-search-path))))
	(if file
	    (find-file file)
	  (error "File not found"))))))