;; jbank@alum.mit.edu
;; 10/2000
;;
;; Provide some improved handling of tcl used with ACS.
;;
;; Exports: acs-tcl-header, ad-proc, proc-doc
;; Modifies tcl font lock keywords.
;; Automatically inserts acs-tcl-header in a new tcl file.

(custom-set-variables
 '(user-mail-address (concat (user-login-name) "@arsdigita.com")))

;; define standard ACS header functions
(defun acs-tcl-header()
  (interactive)
  (insert "# ")
  (if (not (null (buffer-file-name))) 
      (progn (if (string-match "^/web/[^/]*/" (buffer-file-name))
                 (insert "/" (replace-match "" nil nil (buffer-file-name)))
               (insert (buffer-file-name)))
             (if (or (string-match ".*-init.tcl" (buffer-file-name)) (string-match ".*-procs.tcl" (buffer-file-name)))
                 (progn (insert "
ad_library {
     ")
                        (save-excursion (insert "
     @author " user-mail-address " [" user-mail-address "]
     @creation-date " (current-time-string) "
     @cvs-id
}
")))
               (progn (insert "
ad_page_contract {
     ")
                      (save-excursion (insert "
     @author " user-mail-address " [" user-mail-address "]
     @creation-date " (current-time-string) "
     @cvs-id
} {
}
")))))))

(defun ad-proc (proc-name)
  (interactive  (list (read-from-minibuffer "Proc name: ")))
  (insert "# @author " user-mail-address " [" user-mail-address "]
# @creation-date " (current-time-string) "
ad_proc " proc-name " { ")
  (save-excursion (insert "} { Dummy comment.} {

}
")))


(defun proc-doc (proc-name)
  (interactive  (list (read-from-minibuffer "Proc name: ")))
  (insert "# @author " user-mail-address " [" user-mail-address "]
# @creation-date " (current-time-string) "
proc_doc " proc-name " { ")
  (save-excursion (insert "} { Dummy comment.} {

}
")))



(add-hook 'tcl-mode-hook
   (function (lambda ()
        (font-lock-mode)
        (if (zerop (buffer-size))
     (progn
       (acs-tcl-header))))))


;;;Arsdigita tcl extensions
(add-hook 'tcl-mode-hook
          (function (lambda ()
                      (setq tcl-font-lock-keywords
                            (list
                             ;;
                             ;; Function name declarations.
                             '("\\<\\(itcl_class\\|method\\|proc\\|ad_proc\\|proc_doc\\)\\>[ \t]*\\(\\sw+\\)?"
                               (1 font-lock-keyword-face) (2 font-lock-function-name-face nil t))
                             ;;
                             ;; Keywords.
                                        ;(make-regexp '("if" "then" "else" "elseif" "for" "foreach" "break"
                                        ;	       "continue" "while" "eval" "case" "in" "switch" "default"
                                        ;	       "exit" "error" "proc" "return" "uplevel" "constructor"
                                        ;	       "destructor" "itcl_class" "loop" "for_array_keys"
                                        ;	       "for_recursive_glob" "for_file"))
                             (concat "\\<\\("
                                     "break\\|c\\(ase\\|on\\(structor\\|tinue\\)\\)\\|"
                                     "de\\(fault\\|structor\\)\\|"
                                     "e\\(lse\\(\\|if\\)\\|rror\\|val\\|xit\\)\\|"
                                     "for\\(\\|_\\(array_keys\\|file\\|recursive_glob\\)\\|each\\)\\|"
                                     "i\\([fn]\\|tcl_class\\)\\|loop\\|proc\\|return\\|switch\\|"
                                     "then\\|uplevel\\|while"
                                     "\\)\\>")
                             ;;
                             ;; Types.
                                        ;   (make-regexp '("global" "upvar" "inherit" "public" "protected" "common"))
                             (cons (concat "\\<\\(common\\|global\\|inherit\\|"
                                           "p\\(rotected\\|ublic\\)\\|upvar\\)\\>")
                                   'font-lock-type-face)
                             ))
                      (setq tcl-imenu-generic-expression
                            '((nil "^[ \t]*(proc\\|ad_proc\\|proc_doc)[ \t]+\\(\\(\\s_\\|\\sw\\)+\\)" 1))))))

;;;;;
;;bindings for commands
;;C-c a  tries to find the corresponding .adp file for a .tcl file
;;C-c t  tries to find the corresponding .tcl file for a .adp file
(autoload 'multi-mode
  "multi-mode"
  "Allowing multiple major modes in a buffer."
  t)

(defun adp-mode () (interactive)
  (multi-mode 1
              'html-mode
              '("<%" tcl-mode)
              '("%>" html-mode)))

(setq auto-mode-alist
      (append '(("\\.adp$" . adp-mode))
            auto-mode-alist))

(global-set-key "\C-ca" 'tcl-to-adp)
(global-set-key "\C-ct" 'adp-to-tcl)

(if (boundp 'tcl-mode-map)
    (define-key tcl-mode-map "\M-\C-q" 'indent-sexp))



;;paths that are searched for the files
(defvar tcl-search-path (list "./"))

;;to dynamically add to the search path, the hook locate-file-fail-hook
;;can be used: for example the following prompts for a file interactively when
;;the search failes
(defun locate-file-fail-hook ()
  (call-interactively 'find-file))

;;some utilities
(defun has-suffix (suff &optional name)
  (string-match suff (or name (buffer-name))))

(defun no-suffix ( &optional name)
  (substring (or name (buffer-name))
		     0 (string-match "\\." (or name (buffer-name)))))

;;for example change from the oldsuff to the newsuf version of the file
(defun change-to-suff (oldsuff newsuff)
  (interactive)
  (if (not (has-suffix oldsuff))
      (error (concat "Not a " oldsuff " file"))
    ;;search both upcase and downcase suffix
    (let ((file (locate-file (no-suffix)
			     tcl-search-path
			     (concat ":" (upcase newsuff) ":" (downcase newsuff) ":"))))
      (if file
	  (find-file file)
	(if (fboundp 'locate-file-fail-hook)
	    (locate-file-fail-hook))))))

(defun tcl-to-adp ()
  (interactive)
  (change-to-suff ".tcl" ".adp"))

(defun adp-to-tcl ()
  (interactive)
  (change-to-suff ".adp" ".tcl"))
