;; jbank@alum.mit.edu
;; 10/2000
;;
;; Load my preferences.  This is for things like my colors, key bindings,
;; font lock settings, etc.  Things that provide functionality shouldn't
;; go here.
;;
;; requires: user-email, user-full-name


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Font lock settings.  Try to deal with different emacs versions smoothly.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq font-lock-global-modes  '(c-mode c++-mode lisp-mode java-mode emacs-lisp html-mode sh-mode sql-mode perl-mode makefile-mode tcl-mode jsp-mode))
(if (fboundp 'global-font-lock-mode)
     (global-font-lock-mode t))

;; Maximum colors
(setq font-lock-maximum-decoration t)

(setq font-lock-face-attributes
      '((font-lock-comment-face "Red")
        (font-lock-string-face "LightSalmon")
        (font-lock-keyword-face "LightSteelBlue")
        (font-lock-function-name-face "LightSkyBlue")
        (font-lock-variable-name-face "LightGoldenrod")
        (font-lock-type-face "PaleGreen")
        (font-lock-reference-face "Aquamarine")))

(if (fboundp 'custom-set-variables)
    (custom-set-variables))
(if (fboundp 'custom-set-faces)
    (custom-set-faces
     '(font-lock-comment-face ((t (:foreground "Red"))))
     '(font-lock-string-face ((t (:foreground "LightSalmon"))))
     '(font-lock-keyword-face ((t (:foreground "LightSteelBlue"))))
     '(font-lock-function-name-face ((t (:foreground "LightSkyBlue"))))
     '(font-lock-variable-name-face ((t (:foreground "LightGoldenrod"))))
     '(font-lock-type-face ((t (:foreground "PaleGreen"))))
     '(font-lock-reference-face ((t (:foreground "Aquamarine"))))
     ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Load sql modules
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(if (and (> emacs-major-version 19) 
         (or (> emacs-major-version 20) (> emacs-minor-version 2)))
    (progn 
      (autoload 'sql-mode "sql" "SQL mode." t)
      ;;(require 'sql)
      (load "sql")
      (load "sqlmode"))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Display things the way I like them.  This may not be for everyone!!!!
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;Turn off the stupid scroll bars
(if (fboundp 'scroll-bar-mode)
    (scroll-bar-mode -1))
(defadvice new-frame (after scroll-bar-mode activate)
  (scroll-bar-mode -1))

(setq default-frame-alist
           '(;(top . 0) (left . 0)
             (width . 105) (height . 75)
             (cursor-color . "yellow")
             (cursor-type . box)
             (foreground-color . "green")
             (background-color . "black")
             (menu-bar-lines . 0)
             ;(font . "-*-Lucida Console-normal-r-*-*-11-82-*-*-c-*-*-ansi-")
	     ))

; (setq initial-frame-alist
;            '((top . 5) (left . -1)
;              (width . 105) (height . 75)
;              ))
;;(set-frame-height (current-frame) 70)

(setq visible-bell t)
;;; display the time and the date....
(display-time)
(setq display-time-day-and-date t)
(setq display-time-24hr-format t) 

(setq frame-title-format 
      '(multiple-frames ("" invocation-name "@" system-name ": \"%b\"")
                       ("" invocation-name "@" system-name ": \"%b\"")))

(if (fboundp 'colorize) (colorize))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setting for various major modes and setting up modes based on file extensions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq auto-mode-alist 
      (append '(("\\.c$" . c++-mode) ("\\.h$" . c++-mode) ("\\.jsb$" . sh-mode)
                ("\\.properties$" . sh-mode)
                ("\\.sql$" . sql-mode))
	      auto-mode-alist))

(setq completion-ignored-extensions
      (append '(".bci" ".bin" ".binf" ".com" ".ext" ".free" ".kbin"
		".o" ".s" ".rtl" ".ortl" ".nrtl" ".srtl" ".lap" ".kmp")
	      completion-ignored-extensions))
(delete ".log" completion-ignored-extensions)

;; Turn on stuff automatically for various modes
(add-hook 'latex-mode-hook (function (lambda ()
                                      (turn-on-auto-fill)
                                      (setq abbrev-mode 1))))
(add-hook 'tex-mode-hook (function (lambda ()
                                      (turn-on-auto-fill)
                                      (setq abbrev-mode 1))))


(add-hook 'java-mode-hook 'turn-on-font-lock)
(add-hook 'c-mode-hook 'turn-on-font-lock)
(add-hook 'lisp-mode-hook 'turn-on-font-lock)
(add-hook 'emacs-lisp-mode-hook 'turn-on-font-lock)
(add-hook 'latex-mode-hook 'turn-on-font-lock)
(add-hook 'tex-mode-hook 'turn-on-font-lock)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Tell emacs that I can use advanced commands
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(put 'narrow-to-region 'disabled nil)
(put 'downcase-region 'disabled nil)
(setq teach-extended-commands-p nil)
(put 'eval-expression 'disabled nil)
(put 'upcase-region 'disabled nil)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Modify a couple of key bindings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(global-set-key "\C-c\;" 'shell)
(global-set-key "\C-c\C-c" 'comment-region)

(require 'sh-script)
(define-key sh-mode-map "\C-c\C-c" 'comment-region)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Enforce Frequent Type Breaks.  This should help with carpal tunnel problems
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(type-break-mode)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Misc settings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq explicit-shell-file-name "bash")
(setq mail-self-blind t)
(if (get-buffer "*scratch*")		; only buffer so not killed
    (rename-buffer "*-scratch-*"))

;;Setup tabbing
(setq-default indent-tabs-mode nil)


(defun myresync (dir)
  ""
  (interactive (list (read-file-name "Current dir: ")))
  (shell-cd dir))

;;;Mail stuff under Windows

(if (string-match "windows" (symbol-name system-type))
    (progn
      (setq explicit-shell-file-name "c:/cygwin/bin/bash")
      (setq user-full-name userfullname)
      (setq user-mail-address useremail)
      (setq smtpmail-default-smtp-server "mail.arsdigita.com")
      (setq smtpmail-local-domain nil)
      (setq send-mail-function 'smtpmail-send-it)
      (load-library "smtpmail")
      ))



;;example of how to mail my .emacs file to myself
;(defun foo ()
;  (interactive)
;  (mail nil "jbank@alum.mit.com" "Testing script")
;  (insert-file "c:\\.emacs")
;  (mail-send-and-exit)
;  )

;;note, example batch command line
;;emacs -batch -l hack-c >& log

;(require 'gnuserv)
;(gnuserv-start)

(setenv "MANPATH" (concat (if (getenv "MANPATH") (concat (getenv "MANPATH") ":") "")
                        "/usr/local/man:/usr/share/man:/usr/man:/usr/openwin/share/man:/usr/dt/share/man"))